/***************************************************************************
 *   Copyright (C) 2003-2005 by Diego "Flameeyes" Pettenò                  *
 *   flameeyes@users.berlios.de                                            *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/
#include "../version.h"
 
#include "mainwindow.h"

#include <kapplication.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kdebug.h>

#include <SDL.h>

static const char description[] =
    I18N_NOOP("A VisualBoy Advance Frontend");

static const char version[] = VERSION;

static KCmdLineOptions options[] =
{
    KCmdLineLastOption
};

int main(int argc, char **argv)
{
	if(SDL_Init(SDL_INIT_TIMER|SDL_INIT_VIDEO|SDL_INIT_NOPARACHUTE)) {
		kdFatal() << I18N_NOOP("Failed to initialize SDL") << ": " << SDL_GetError() << endl;
		return -1;
	}

	if(SDL_InitSubSystem(SDL_INIT_JOYSTICK)) {
		kdFatal() << I18N_NOOP("Failed to initialize joystick") << ": " << SDL_GetError() << endl;
		return -2;
	}

	int numJoy = SDL_NumJoysticks();

	for(register int i = 0; i < numJoy; i++)
		SDL_JoystickOpen(i);

	SDL_JoystickEventState(SDL_ENABLE);

	KAboutData about("kvba", I18N_NOOP("KVBA"), version, description,
		KAboutData::License_GPL, "(C) 2003-2005 Diego Pettenò", 0, "http://flameeyes.web.ctonet.it/");
	about.addAuthor( "Diego Pettenò", "http://flameeyes.web.ctonet.it/", "flameeyes@users.berlios.de");
	KCmdLineArgs::init(argc, argv, &about);
	KCmdLineArgs::addCmdLineOptions( options );
	KApplication app;
	MainWindow *mainWin = 0;

	if (app.isRestored())
	{
		RESTORE(MainWindow);
	} else {
		// no session.. just start up normally
		KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

		mainWin = new MainWindow();
		app.setMainWidget( mainWin );
		mainWin->show();
	
		args->clear();
	}

	int ret = app.exec();

	SDL_Quit();
	return ret;
}

